<?xml version='1.0' encoding='iso-8859-1'?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd" >

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" >
<head>
<title>{ $TITLE }</title>
<link rel="stylesheet" href="/gallery.css" type="text/css" />
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
{ $META }
<!--[if gte IE 5.5000]>
<link rel="stylesheet" href="/galleryIE.css" type="text/css" />
<![endif]-->
</head>
<body>
{ $MAIN }
</body>
</html>
