<div id="page">
  <div id="title">
    { $MENU }
  </div>

  <div id="menu">
{ $BACK } - <a href="./" accesskey=u" rel="up" title="Accesskey U"><u>u</u>p</a> - { $NEXT }</div>
  <div>
    <table>
      <tr>

        <td colspan="3" id="picture">
          IMG { $NUMBER } of { $TOTAL } { $EXIFVALUES } <br><br>
          <img src="{ $SRC }" alt="* Image { $NUMBER }"><br><br>
[ Size: { $SIZES } | Slideshow: { $SLIDESHOW } ]</td>

              </tr>
              { $PICTUREINFO }
      </tr>
	  <tr>
	  	<td colspan="3">
		  <div id="gallery">
		  <a href="http://apachegallery.dk/">Apache::Gallery</a> &copy; 2001-2005 Michael Legart, <a href="http://www.hestdesign.com/">Hest Design</a>

		  </div>
		</td>
	  </tr>
    </table>
  </div>
</div>
