<div class="gallery-info">
{ $MENU }
</div>
<div id="gallery-nav">{ $BROWSELINKS }</div>
<div class="clr" />
<div id="img-info">
{ $FORM_BEGIN }
</div>
<table>
  <tr>
{ $PREV_DIR_FILES }
    <td id="directory">
      <div>
{ $FILES }
      </div>
    </td>
{ $NEXT_DIR_FILES }
  </tr>
</table>
  { $DIRCOMMENT }
<div class="clr" />
<div class="aginfo">
  		Indexed by <a href="http://apachegallery.dk">Apache::Gallery</a> - Copyright &copy; 2001-2008 Michael Legart - <a href="http://www.hestdesign.com/">Hest Design!</a>
</div>
{ $FORM_END }
