              <tr>
                 <td colspan="3">
                   <div id="comment">
                      { $INFO }
                      { $COMMENT }
                   </div>
                 </td>
              </tr>
